@@ -30,6 +30,7 @@ public:
     virtual shared_ptr<Model> copy() const override;
     virtual const char *type() const override;
     virtual vector<shared_ptr<Model>> slice_to_grid(const tfloat tile_size) const override;
+    shared_ptr<Model> slice_to_rect(const tfloat lower_x, const tfloat lower_y, const tfloat upper_x, const tfloat upper_y) const;
     virtual void map(const shared_ptr<TriangularMesh> target2D) override;
 
     const tvec3 * triangle(const size_t index) const;
@@ -37,6 +38,7 @@ public:
 
     virtual size_t to_obj(const string & path, const size_t offset) const override;
 protected:
+    void to_model(const vector<tvec3> &triangles, shared_ptr<TriangularMesh> model) const;
     void to_tiles(const vector<tvec3> &triangles, const tfloat tile_size, Tiles &tiles) const;
     
     friend class RTree;
